"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _core = require("@babel/core");

var _uglifyJs = _interopRequireDefault(require("uglify-js"));

var _util = _interopRequireDefault(require("./util"));

var _cache = _interopRequireDefault(require("./cache"));

const currentPath = _util.default.currentDir;

const modulesPath = _path.default.join(currentPath, 'node_modules' + _path.default.sep);

const src = _cache.default.getSrc();

const dist = _cache.default.getDist();

let npmPath = _path.default.join(currentPath, dist, 'npm' + _path.default.sep);

var _default = {
  getPkgConfig(lib) {
    let pkg = _util.default.readFile(_path.default.join(modulesPath, lib, 'package.json'));

    try {
      pkg = JSON.parse(pkg);
    } catch (e) {
      pkg = null;
    }

    return pkg;
  },

  resolveDeps(code, opath, status, config) {
    return code.replace(/require\(['"]([\w\d_\-./@]+)['"]\)/ig, (match, lib) => {
      let resolved = lib;
      let target = '';
      let source = '';
      let ext = '';
      let needCopy = false;

      if (lib[0] === '.') {
        // require('./something'');
        source = _path.default.join(opath.dir, lib); // e:/src/util

        if (status === 'npm') {
          target = _path.default.join(npmPath, _path.default.relative(modulesPath, source));
          needCopy = true;
        } else {
          // e:/dist/util
          target = _util.default.getDistPath(source);
          needCopy = false;
        }
      } else if (lib.indexOf('/') === -1 || // require('asset')
      lib.indexOf('/') === lib.length - 1 || // reqiore('a/b/something/')
      lib[0] === '@' && lib.indexOf('/') !== -1 && lib.lastIndexOf('/') === lib.indexOf('/') // require('@abc/something')
      ) {
          let pkg = this.getPkgConfig(lib);

          if (!pkg) {
            _util.default.error('找不到模块: ' + lib);
          }

          let main = pkg.main || 'index.js';

          if (pkg.browser && typeof pkg.browser === 'string') {
            main = pkg.browser;
          }

          source = _path.default.join(modulesPath, lib, main);
          target = _path.default.join(npmPath, lib, main);
          lib += _path.default.sep + main;
          ext = '';
          needCopy = true;
        } else {
        // require('babel-runtime/regenerator')
        source = _path.default.join(modulesPath, lib);
        target = _path.default.join(npmPath, lib);
        ext = '';
        needCopy = true;
      }

      if (_util.default.isFile(source + '.js')) {
        ext = '.js';
      } else if (_util.default.isFile(source + '.js')) {
        ext = '.js';
      } else if (_util.default.isDir(source) && _util.default.isFile(source + _path.default.sep + 'index.js')) {
        ext = _path.default.sep + 'index.js';
      } else if (_util.default.isFile(source)) {
        ext = '';
      } else {
        _util.default.error('找不到文件: ' + source);
      }

      source += ext;
      target += ext;
      lib += ext;
      resolved = lib;

      if (needCopy) {
        if (!_cache.default.checkBuildCache(source)) {
          _cache.default.setBuildCache(source);

          this.compile(null, _path.default.parse(source), 'npm', config);
        }
      }

      if (status === 'npm') {
        if (lib[0] !== '.') {
          resolved = _path.default.join('..' + _path.default.sep, _path.default.relative(opath.dir, modulesPath), lib);
        } else {
          if (lib[0] === '.' && lib[1] === '.') {
            resolved = './' + resolved;
          }
        }
      } else {
        resolved = _path.default.relative(_util.default.getDistPath(opath, opath.ext, src, dist), target);
      }

      resolved = resolved.replace(/\\/g, '/').replace(/^\.\.\//, './');
      return `require('${resolved}')`;
    });
  },

  npmHack(filename, code) {
    // 一些库（redux等） 可能会依赖 process.env.NODE_ENV 进行逻辑判断
    // 这里在编译这一步直接做替换 否则报错
    const config = _util.default.getConfig();

    code = code.replace(/process\.env\.NODE_ENV/g, `'${config.env || "development"}'`);

    switch (filename) {
      case 'lodash.js':
      case '_global.js':
        code = code.replace(/Function(\s*'\s*return this\s*'\s*)()/, 'this');
        break;

      case '_html.js':
        code = 'module.exports = false;';
        break;

      case '_microtask.js':
        code = code.replace(/if\s*\(\s*Observer\s*\)/, 'if(false && Observer)'); // IOS 1.10.2 Promise BUG

        code = code.replace(/Promise\s*&&\s*Promise\.resolve/, 'false && Promise && Promise.resolve');
        break;

      case '_freeGlobal.js':
        code = code.replace(/module\.exports\s*=\s*freeGlobal/, 'module.exports = freeGlobal || this');
    }

    return code;
  },

  compile(content, opath, status, config) {
    let target;

    let filepath = _path.default.join(opath.dir, opath.base);

    if (!content) {
      content = _util.default.readFile(filepath) || '';

      if (content === null) {
        _util.default.warning('打开文件失败: ' + _path.default.join(opath.dir, opath.base));
      }
    }

    if (status === 'npm') {
      content = {
        code: content
      };
    } else {
      const config = _util.default.getConfig();

      content = content.replace(/process\.env\.NODE_ENV/g, `'${config.env || "development"}'`);
      content = (0, _core.transformSync)(content, {
        sourceMap: config.env !== 'production'
      });
    }

    let replace = '';

    if (status === 'app' || status === 'page' || status === 'component') {
      content.code = content.code.replace(/exports\.default\s*=\s*((\w+);)/ig, (m, b, defaultExport) => {
        if (defaultExport === 'undefined') {
          return '';
        }

        if (status === 'app') {
          replace = `\nApp(require('wemix').default.$createApp(${defaultExport}));\n`;
        } else if (status === 'page') {
          let pagePath = _path.default.join(_path.default.relative(_path.default.join(_util.default.currentDir, src), opath.dir), opath.name).replace(/\\/ig, '/');

          replace = `\nPage(require('wemix').default.$createPage(${defaultExport} , '${pagePath}'));\n`;
        } else if (status === 'component') {
          let pagePath = _path.default.join(_path.default.relative(_path.default.join(_util.default.currentDir, src), opath.dir), opath.name).replace(/\\/ig, '/');

          replace = `\nComponent(require('wemix').default.$createComponent(${defaultExport} , '${pagePath}'));\n`;
        }

        return '';
      });
      content.code += replace;
    }
    /**
     * 转换 foobar instanceof Function 为 typeof foobar ==='function'
     * 由于微信重定义了全局的Function对象，所以moment等npm库会出现异常
     */


    content.code = content.code.replace(/([\w[\]a-d.]+)\s*instanceof Function/g, function (matchs, word) {
      return ' typeof ' + word + " ==='function' ";
    });
    content.code = this.resolveDeps(content.code, opath, status, config);

    if (status !== 'npm') {
      target = _util.default.getDistPath(opath, opath.ext, src, dist);
    } else {
      content.code = this.npmHack(opath.base, content.code);
      target = _path.default.join(npmPath, _path.default.relative(modulesPath, _path.default.join(opath.dir, opath.base)));
    }

    if (content.map) {
      content.map.sources = [opath.name + '.js'];
      content.map.file = opath.name + '.js';

      var Base64 = require('js-base64').Base64;

      content.code += `\r\n//# sourceMappingURL=data:application/json;charset=utf-8;base64,${Base64.encode(JSON.stringify(content.map || {}))}`;
    }

    if (config.env === 'production') {
      let rst = _uglifyJs.default.minify(content.code, {
        compress: {
          warnings: false
        }
      });

      content.code = rst.code;
    }

    _util.default.output('编译', filepath);

    _util.default.output('写入', target);

    _util.default.writeFile(target, content.code);

    _cache.default.saveBuildCache();
  }

};
exports.default = _default;