"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _less = _interopRequireDefault(require("less"));

var _path = _interopRequireDefault(require("path"));

var _https = _interopRequireDefault(require("https"));

var _postcss = _interopRequireDefault(require("postcss"));

var _cssnano = _interopRequireDefault(require("cssnano"));

var _postcssPresetEnv = _interopRequireDefault(require("postcss-preset-env"));

var _util = _interopRequireDefault(require("./util"));

var _cache = _interopRequireDefault(require("./cache"));

const src = _cache.default.getSrc();

const dist = _cache.default.getDist();

const grabConfigFromScript = (str, n) => {
  let stash = [];
  let rst = '';

  for (let i = n, l = str.length; i < l; i++) {
    if (str[i] === '{') {
      stash.push('{');
    }

    if (str[i] === '}') {
      stash.pop();

      if (stash.length === 0) {
        rst += '}';
        break;
      }
    }

    if (stash.length) {
      rst += str[i];
    }
  }

  return rst;
};

const hackIconfont = lessContent => {
  return new Promise((resolve, reject) => {
    if (lessContent.indexOf('@iconfont') >= 0) {
      lessContent.replace(/@iconfont\:\s*['"](.+)['"];/, (replace, url) => {
        if (url.indexOf('//') === 0) {
          url = 'https:' + url;
        } else if (url.indexOf('http:') === 0) {
          url = url.replace('http:', 'https:');
        }

        _https.default.get(url, res => {
          let content = '';
          res.on('error', err => {
            reject(err);
          });
          res.on('data', chunk => {
            content += chunk;
          });
          res.on('end', () => {
            let match = content.match(new RegExp(`[\\s\\r\\n]\@font-face[\\s\\r\\n]*`));
            match = match ? match[0] : undefined;
            let replaceContent = match ? grabConfigFromScript(content, content.indexOf(match) + match.length) : false;
            replaceContent.replace(/url\(['"]([\s\S]+?)['"]\)/, (c, url) => {
              if (url.indexOf('//') === 0) {
                url = 'https:' + url;
              } else if (url.indexOf('http:') === 0) {
                url = url.replace('http:', 'https:');
              }

              url = url.substr(0, url.lastIndexOf('.')) + '.ttf?t=' + Date.now();

              _https.default.get(url, res => {
                let chunks = [];
                res.on('error', err => {
                  reject(err);
                });
                res.on('data', chunk => {
                  chunks.push(chunk);
                });
                res.on('end', () => {
                  let base64 = Buffer.concat(chunks).toString('base64');
                  let font = `{font-family: 'iconfont';src: url(data:font/truetype;charset=utf-8;base64,#BASE64) format('truetype');font-weight: normal;font-style: normal;}`;
                  font = font.replace('#BASE64', base64);
                  content = content.replace(replaceContent, font);
                  lessContent = lessContent.replace(replace, content);
                  lessContent = lessContent.replace(/(\d+(\.{0,1}\d+){0,1})px/, (a, b, c, d) => {
                    return a.replace(a, `${_util.default.accMul(b, 2)}rpx`);
                  });
                  resolve(lessContent);
                });
              });
            });
          });
        });
      });
    } else {
      resolve(lessContent);
    }
  });
};

var _default = {
  compile(opath, config) {
    let filepath = _path.default.join(opath.dir, opath.base);

    let target = _util.default.getDistPath(opath, 'wxss', src, dist);

    let content = _util.default.readFile(filepath) || '';
    hackIconfont(content).then(formatContent => {
      let cssPlugins = [(0, _postcssPresetEnv.default)({
        stage: 0,
        browsers: ['Chrome >= 35', 'ChromeAndroid >= 35', 'iOS >= 8', 'Android >= 4.1']
      })];
      let conf = {
        paths: [opath.dir]
      };

      if (config.env === 'production') {
        cssPlugins.push((0, _cssnano.default)({
          preset: 'default',
          autoprefixer: false,
          reduceIdents: false,
          'postcss-zindex': false
        }));
      }

      let imports = [];
      let hasImport = true;

      while (hasImport) {
        let length = 0;

        let _import = formatContent.substring(formatContent.indexOf('@import'), formatContent.indexOf('.less') + 5);

        if (_import.indexOf('@import') > -1 && _import.indexOf('.less') > -1) {
          length = _import.length;
          let close = formatContent.substr(formatContent.indexOf('.less') + 5, 1);

          if (close === '\'' || close === '\"') {
            _import += close;
            length++;
            let end = formatContent.substr(formatContent.indexOf('.less') + 6, 1);

            if (end === ';') {
              length++;
              _import += end;
            }
          }

          formatContent = formatContent.substr(length + 1);
          imports.push(_import.replace('.less', '.wxss'));
        } else {
          hasImport = false;
        }
      }

      _less.default.render(formatContent, conf).then(res => {
        (0, _postcss.default)(cssPlugins).process(res.css, {
          from: filepath,
          to: target
        }).then(result => {
          result.css = imports.join('\n') + (imports.length ? '\n' + result.css : result.css);

          _util.default.output('编译', filepath);

          _util.default.output('写入', target);

          _util.default.writeFile(target, result.css);

          if (result.map) {
            _util.default.writeFile(`${target}.map`, result.map);
          }

          _cache.default.saveBuildCache();
        });
      }).catch(error => {
        _util.default.error(JSON.stringify(error));
      });
    }).catch(e => {
      _util.default.error('更新iconfont错误: ' + e);
    });
  }

};
exports.default = _default;