"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _util = _interopRequireDefault(require("./util"));

var _cache = _interopRequireDefault(require("./cache"));

var _loader = _interopRequireDefault(require("./loader"));

let watchReady = false;
let preventDup = {};
var _default = {
  watch(config) {
    config.watch = false;

    let src = _cache.default.getSrc();

    _chokidar.default.watch(`.${_path.default.sep}${src}`, {
      depth: 99
    }).on('all', (evt, filepath) => {
      if ((evt === 'change' || evt === 'add') && watchReady && !preventDup[filepath]) {
        preventDup[filepath] = evt;
        config.file = _path.default.join('..', filepath);

        _util.default.log('文件: ' + filepath, '变更');

        this.build(config);
        setTimeout(() => {
          preventDup[filepath] = false;
        }, 500);
      }
    }).on('ready', () => {
      watchReady = true;

      _util.default.log('开始监听文件改动。', '信息');
    });
  },

  build(config) {
    let src = _cache.default.getSrc();

    let dist = _cache.default.getDist();

    let current = process.cwd();

    let files = _util.default.getFiles(src);

    let wemixConfig = _util.default.getConfig(config);

    if (!_util.default.isDir(_path.default.join(_util.default.currentDir, dist, 'npm', 'wemix'))) {
      config.cache = false;
    }

    if (!config.cache || wemixConfig.env === 'production') {
      _cache.default.clearBuildCache();
    }

    if (files.some(v => v === 'app.js')) {
      // 如果编译文件包含app.js，且第一个不是 app.js
      if (_util.default.isFile(_path.default.join(current, src, 'app.js'))) {
        // src/app.js 存在, 则把它放在第一位, 因为后面需要取页面路径
        let newFiles = ['app.js'].concat(files.filter(v => {
          return v !== 'app.js' && v.indexOf('.DS_Store') === -1;
        }));
        files = newFiles;
      } else {
        _util.default.error('根目录不存在app.js');
      }
    }

    files.forEach(f => {
      let opath = _path.default.parse(_path.default.join(current, src, f));

      this.compile(opath);
    });

    if (config.watch) {
      this.watch(config);
    }
  },

  compile(opath) {
    if (!_cache.default.checkBuildCache(_path.default.join(opath.dir, opath.base))) {
      _cache.default.setBuildCache(_path.default.join(opath.dir, opath.base));

      _loader.default.compile(opath);
    }
  }

};
exports.default = _default;