"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _safe = _interopRequireDefault(require("colors/safe"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _child_process = require("child_process");

var _rimraf = _interopRequireDefault(require("rimraf"));

var _cache = _interopRequireDefault(require("./cache"));

_safe.default.enabled = true;

_safe.default.setTheme({
  'info': 'grey',
  '变更': 'bgYellow',
  '删除': 'bgMagenta',
  '执行': 'blue',
  '压缩': 'blue',
  '信息': 'grey',
  '完成': 'green',
  '创建': 'green',
  '监听': 'magenta',
  '错误': 'red',
  '测试': 'red',
  '拷贝': 'yellow',
  '编译': 'blue',
  '写入': 'green'
});

const utils = {
  isFunction(fn) {
    return typeof fn === 'function';
  },

  isString(obj) {
    return toString.call(obj) === '[object String]';
  },

  isObject(obj) {
    return toString.call(obj) === '[object Object]';
  },

  isNumber(obj) {
    return toString.call(obj) === '[object Number]';
  },

  isBoolean(obj) {
    return toString.call(obj) === '[object Boolean]';
  },

  isArray(obj) {
    return Array.isArray(obj);
  },

  isFile(p) {
    p = typeof p === 'object' ? _path.default.join(p.dir, p.base) : p;

    if (!_fs.default.existsSync(p)) {
      return false;
    }

    return _fs.default.statSync(p).isFile();
  },

  isDir(p) {
    if (!_fs.default.existsSync(p)) {
      return false;
    }

    return _fs.default.statSync(p).isDirectory();
  },

  currentDir: process.cwd(),
  cliDir: __dirname,

  getVersion() {
    let filepath = _path.default.resolve(__dirname, '../package.json');

    let version = JSON.parse(this.readFile(filepath)).version;
    return version;
  },

  getConfig(conf) {
    let config = _cache.default.getConfig();

    if (config) {
      return config;
    } else {
      config = conf;

      _cache.default.setConfig(config);

      return config;
    }
  },

  readFile(p) {
    let rst = '';
    p = typeof p === 'object' ? _path.default.join(p.dir, p.base) : p;

    try {
      if (p.lastIndexOf('.jpg') > -1 || p.lastIndexOf('.jpeg') > -1 || p.lastIndexOf('.png') > -1 || p.lastIndexOf('.gif') > -1) {
        rst = _fs.default.readFileSync(p);
      } else {
        rst = _fs.default.readFileSync(p, 'utf-8');
      }
    } catch (e) {
      rst = null;
    }

    return rst;
  },

  writeFile(p, data) {
    let opath = this.isString(p) ? _path.default.parse(p) : p;

    if (!this.isDir(opath.dir)) {
      _mkdirp.default.sync(opath.dir);
    }

    _fs.default.writeFileSync(p, data);
  },

  getFiles(dir = process.cwd(), prefix = '') {
    let cfiles = _cache.default.getFileList(dir);

    if (cfiles) {
      return cfiles;
    }

    dir = _path.default.normalize(dir);

    if (!_fs.default.existsSync(dir)) {
      return [];
    }

    let files = _fs.default.readdirSync(dir);

    let rst = [];
    files.forEach(item => {
      let filepath = dir + _path.default.sep + item;

      let stat = _fs.default.statSync(filepath);

      if (stat.isFile()) {
        rst.push(prefix + item);
      } else if (stat.isDirectory()) {
        rst = rst.concat(this.getFiles(_path.default.normalize(dir + _path.default.sep + item), _path.default.normalize(prefix + item + _path.default.sep)));
      }
    });

    _cache.default.setFileList(dir, rst);

    return rst;
  },

  getDistPath(opath, ext, src, dist) {
    let relative;
    src = src || _cache.default.getSrc();
    dist = dist || _cache.default.getDist();

    if (typeof opath === 'string') {
      opath = _path.default.parse(opath);
    }

    ext = ext ? ext[0] === '.' ? ext : '.' + ext : opath.ext; // 第三组件
    // 相对目录以node_modules开始

    if (_path.default.relative(this.currentDir, opath.dir).indexOf('node_modules') === 0) {
      relative = _path.default.relative(_path.default.join(this.currentDir, 'node_modules'), opath.dir);
      relative = _path.default.join('npm', relative);
    } else {
      relative = _path.default.relative(_path.default.join(this.currentDir, src), opath.dir);
    }

    return _path.default.join(this.currentDir, dist, relative, opath.name + ext);
  },

  output(type, file, flag) {
    if (!flag) {
      flag = file.substr(file.lastIndexOf('.') + 1).toUpperCase();

      if (flag.length < 4) {
        let i = 4 - flag.length;

        while (i--) {
          flag += ' ';
        }
      }
    }

    this.log(flag + ': ' + _path.default.relative(this.currentDir, file), type);
  },

  mkdir(name) {
    let rst = true;

    try {
      _fs.default.mkdirSync(name);
    } catch (e) {
      rst = e;
    }

    return rst;
  },

  error(msg) {
    this.log(msg, 'error', false);
  },

  warning(msg) {
    this.log(msg, 'warning', false);
  },

  log(msg, type, showTime = true) {
    let dateTime = showTime ? _safe.default.gray(`[${this.datetime()}] `) : '';

    if (this.isObject(msg) || this.isArray(msg)) {
      msg = JSON.stringify(msg || {});
    }

    if (type && this.isString(type)) {
      type = type.toUpperCase();

      if (type === 'ERROR') {
        console.error(_safe.default.red('[Error] ' + msg));
      } else if (type === 'WARNING') {
        console.error(_safe.default.yellow('[WARNING] ' + msg));
      } else {
        let fn = _safe.default[type] ? _safe.default[type] : _safe.default['info'];
        console.log(dateTime + fn(`[${type}]`) + ' ' + msg);
      }
    } else {
      console.log(dateTime + msg);
    }
  },

  datetime(date = new Date(), format = 'HH:mm:ss') {
    let fn = d => {
      return ('0' + d).slice(-2);
    };

    if (date && this.isString(date)) {
      date = new Date(Date.parse(date));
    }

    const formats = {
      YYYY: date.getFullYear(),
      MM: fn(date.getMonth() + 1),
      DD: fn(date.getDate()),
      HH: fn(date.getHours()),
      mm: fn(date.getMinutes()),
      ss: fn(date.getSeconds())
    };
    return format.replace(/([a-z])\1+/ig, function (a) {
      return formats[a] || a;
    });
  },

  exec(cmd, quite) {
    return new Promise((resolve, reject) => {
      let fcmd = (0, _child_process.exec)(cmd, (err, stdout, stderr) => {
        if (err) {
          reject(err);
        } else {
          resolve(stdout, stderr);
        }
      });
      fcmd.stdout.on('data', chunk => {
        !quite && process.stdout.write(chunk);
      });
      fcmd.stderr.on('data', chunk => {
        !quite && process.stdout.write(chunk);
      });
    });
  },

  getModifiedTime(p) {
    return this.isFile(p) ? +_fs.default.statSync(p).mtime : 1;
  },

  unlink(p) {
    let rst = '';
    p = typeof p === 'object' ? _path.default.join(p.dir, p.base) : p;

    try {
      rst = _fs.default.unlinkSync(p);
    } catch (e) {
      rst = null;
    }

    return rst;
  },

  clearDist(callback) {
    const proConf = this.readFile(_path.default.join(this.currentDir, 'dist/project.config.json'));
    const jsConf = this.readFile(_path.default.join(this.currentDir, 'dist/jsconfig.json'));
    (0, _rimraf.default)(_path.default.join(this.currentDir, 'dist'), () => {
      proConf && this.writeFile(_path.default.join(this.currentDir, 'dist/project.config.json'), proConf);
      jsConf && this.writeFile(_path.default.join(this.currentDir, 'dist/jsconfig.json'), jsConf);
      callback();
    });
  },

  accMul(arg1, arg2) {
    let m = 0;
    let s1 = arg1.toString();
    let s2 = arg2.toString();

    try {
      m += s1.split('.')[1].length;
    } catch (e) {}

    try {
      m += s2.split('.')[1].length;
    } catch (e) {}

    return Number(s1.replace('.', '')) * Number(s2.replace('.', '')) / Math.pow(10, m);
  }

};
var _default = utils;
exports.default = _default;