"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _commander = _interopRequireDefault(require("commander"));

var _path = _interopRequireDefault(require("path"));

var _util = _interopRequireDefault(require("./util"));

var _compile = _interopRequireDefault(require("./compile"));

const grabConfigFromScript = (str, n, path) => {
  let collect = false;
  let stash = [];
  let rst = '';
  let pages = [];

  for (let i = n, l = str.length; i < l; i++) {
    if (stash.join('') === 'pages') {
      if (str[i] === '[') {
        collect = true;
      }

      if (collect) {
        rst += str[i];
      }

      if (str[i] === ']') {
        break;
      }
    } else {
      if (stash.length === 5) {
        stash.shift();
      }

      stash.push(str[i]);
    }
  }

  try {
    if (rst) {
      pages = new Function(`return ${rst}`)();
      /* eslint-disable-line */
    }
  } catch (e) {
    _util.default.output('错误', path);

    _util.default.error(`解析pages出错，报错信息：${e}\r\n${rst}`);
  }

  return {
    pages,
    rst
  };
};

const displayVersion = () => {
  let version = _util.default.getVersion();

  let chars = ['   __    ____  __  __  _  _   _ ', '( \\/\\/ )(____)(  \\/  )( )( \\_/ )', ' )    (  )__)  )    ( ( ) ) _ ( ', '(__/\\__)(____)( /\\/\\ )(_)(_/ \\_)', '                                         '].join('\n');
  console.log('\n v' + version + '\n');
  console.log(chars);
};

const generateProject = (name, config) => {
  let inPlace = !name || name === '.';

  if (inPlace) {
    name = process.cwd().split(_path.default.sep).pop();

    _util.default.log('使用当前目录：' + name);
  } else {
    _util.default.log('目录：' + name, '创建');

    if (_util.default.mkdir(name) !== true) {
      _util.default.error('创建目录失败。');

      return;
    }

    process.chdir(name);
  }

  _util.default.currentDir = process.cwd();

  let packagePath = _path.default.join(_util.default.currentDir, 'package.json');

  if (_util.default.isFile(packagePath) || _util.default.isDir(_path.default.join(_util.default.currentDir, 'src'))) {
    _util.default.error('目录不为空, 请请勿重复初始化', '错误');

    return;
  }

  const tplDir = '../templates/empty';

  const template = _path.default.join(_util.default.cliDir, tplDir, _path.default.sep);

  const dependencies = ['wemix'];

  let pkg = _path.default.join(template, 'package.json');

  pkg = _util.default.readFile(pkg);
  pkg = JSON.parse(pkg);
  pkg.name = name;

  _util.default.writeFile(packagePath, JSON.stringify(pkg || {}));

  _util.default.log('配置: ' + 'package.json', '写入');

  let files = _util.default.getFiles(template).filter(file => file !== 'package.json');

  const copyFn = function copyFn(sourcePath) {
    return function (file) {
      let target = _path.default.join(_util.default.currentDir, file);

      let fileContent = _util.default.readFile(_path.default.join(sourcePath, file));

      _util.default.writeFile(target, fileContent);

      _util.default.log('文件: ' + file, '拷贝');
    };
  };

  files.forEach(copyFn(template));
  let cmd = 'npm install --registry https://registry.npm.taobao.org';
  let cmdD = `npm install ${dependencies.join(' ')} --save --registry https://registry.npm.taobao.org`;

  _util.default.log('执行命令: ' + cmd, '执行');

  _util.default.log('可能需要几分钟, 请耐心等待...', '信息');

  _util.default.exec(cmd).then(d => {
    return _util.default.exec(cmdD);
  }).then(d => {
    _util.default.log('安装依赖完成', '完成');

    let cmd = 'wemix build';

    _util.default.log('执行命令: ' + cmd, '执行');

    _util.default.log('可能需要几分钟, 请耐心等待...', '信息');

    _util.default.exec(cmd).then(d => {
      _util.default.log('代码编译完成', '完成');

      _util.default.log('项目初始化完成, 可以开始使用小程序。', '完成');
    }).catch(e => {
      _util.default.log('代码编译出错', '错误');
    });
  }).catch(e => {
    _util.default.log('安装依赖出错', '错误');
  });
};

const generatePage = (name, config) => {
  if (!name) {
    return _util.default.log('请输入文件路径');
  }

  if (name.indexOf('components') > 0 || name.indexOf('behaviors') > 0) {
    return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
  }

  let className, target, appPath, pathSplit, appContent;
  let exts = ['js', 'less', 'html'];
  name = name.replace(/(^\s*)|(\s*$)/g, '');

  if (name.indexOf('src') === -1 && name.indexOf('pages') === -1 && name[0] !== '.' && name[0] !== '/') {
    target = _path.default.join(process.cwd(), '/src/pages', name);
    appPath = _path.default.join(process.cwd(), '/src/app.js');
  } else {
    if (name.indexOf('src/pages/') === 0 || name.indexOf('/src/pages/') === 0 || name.indexOf('./src/pages/') === 0) {
      target = _path.default.join(process.cwd(), name);
      appPath = target.substr(0, target.indexOf('src') + 3) + '/app.js';
    } else {
      return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
    }
  }

  pathSplit = target.split(_path.default.sep);
  appContent = _util.default.readFile(appPath);

  if (/index/i.test(pathSplit[pathSplit.length - 1])) {
    if (/pages/i.test(pathSplit[pathSplit.length - 2])) {
      className = 'Index';
    } else {
      className = pathSplit[pathSplit.length - 2].replace(/^\S/, s => {
        return s.toUpperCase();
      });
    }
  } else {
    className = pathSplit[pathSplit.length - 1].replace(/^\S/, s => {
      return s.toUpperCase();
    });
  }

  exts.forEach(item => {
    let fileContent = _util.default.readFile(_path.default.join(_util.default.cliDir, `../templates/page/index.${item}`));

    if (item === 'js') {
      fileContent = fileContent.replace(/___PAGE___/g, className);
    }

    _util.default.writeFile(`${target}.${item}`, fileContent);

    _util.default.log(`文件: ${target}.${item}`, '拷贝');
  });

  _util.default.output('编译', appPath);

  appContent.replace(/export\s*default\s*class\s*(\w+)\s*extends/ig, (m, title) => {
    let match = appContent.match(new RegExp(`[\\s\\r\\n]${title}.config\\s*=[\\s\\r\\n]*`));
    match = match ? match[0] : undefined;
    let json = match ? grabConfigFromScript(appContent, appContent.indexOf(match) + match.length, appPath) : false;
    json.pages.push(target.substr(target.indexOf('pages/')));
    let pagesContent = '[';
    json.pages.forEach(item => {
      pagesContent += `\n    '${item}',`;
    });
    pagesContent = pagesContent.substr(0, pagesContent.length - 1);
    pagesContent += '\n  ]';
    appContent = appContent.replace(json.rst, pagesContent);

    _util.default.output('写入', appPath);

    _util.default.writeFile(appPath, appContent);
  });
};

const generateComponent = (name, config) => {
  if (!name) {
    return _util.default.log('请输入文件路径');
  }

  if (name.indexOf('pages') > 0 || name.indexOf('behaviors') > 0) {
    return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
  }

  let className, target, pathSplit;
  let exts = ['js', 'less', 'html'];
  name = name.replace(/(^\s*)|(\s*$)/g, '');

  if (name.indexOf('src') === -1 && name.indexOf('components') === -1 && name[0] !== '.' && name[0] !== '/') {
    target = _path.default.join(process.cwd(), '/src/components', name);
  } else {
    if (name.indexOf('src/components/') === 0 || name.indexOf('/src/components/') === 0 || name.indexOf('./src/components/') === 0) {
      target = _path.default.join(process.cwd(), name);
    } else {
      return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
    }
  }

  pathSplit = target.split(_path.default.sep);

  if (/index/i.test(pathSplit[pathSplit.length - 1])) {
    if (/components/i.test(pathSplit[pathSplit.length - 2])) {
      className = 'Index';
    } else {
      className = pathSplit[pathSplit.length - 2].replace(/^\S/, s => {
        return s.toUpperCase();
      });
    }
  } else {
    className = pathSplit[pathSplit.length - 1].replace(/^\S/, s => {
      return s.toUpperCase();
    });
  }

  exts.forEach(item => {
    let fileContent = _util.default.readFile(_path.default.join(_util.default.cliDir, `../templates/component/index.${item}`));

    if (item === 'js') {
      fileContent = fileContent.replace(/___COMPONENT___/g, className);
    }

    _util.default.writeFile(`${target}.${item}`, fileContent);

    _util.default.log(`文件: ${target}.${item}`, '拷贝');
  });
};

const generateBehavior = (name, config) => {
  if (!name) {
    return _util.default.log('请输入文件路径');
  }

  if (name.indexOf('pages') > 0 || name.indexOf('components') > 0) {
    return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
  }

  let target;
  let exts = ['js', 'less', 'html'];
  name = name.replace(/(^\s*)|(\s*$)/g, '');

  if (name.indexOf('src') === -1 && name.indexOf('behaviors') === -1 && name[0] !== '.' && name[0] !== '/') {
    target = _path.default.join(process.cwd(), '/src/behaviors', name);
  } else {
    if (name.indexOf('src/behaviors/') === 0 || name.indexOf('/src/behaviors/') === 0 || name.indexOf('./src/behaviors/') === 0) {
      target = _path.default.join(process.cwd(), name);
    } else {
      return _util.default.error('生成路径错误：' + _path.default.join(process.cwd(), name), '错误');
    }
  }

  let fileContent = _util.default.readFile(_path.default.join(_util.default.cliDir, `../templates/behavior/index.js`));

  _util.default.writeFile(`${target}.js`, fileContent);

  _util.default.log(`文件: ${target}.js`, '拷贝');
};

_commander.default.usage('[command] <options ...>');

_commander.default.option('-v, --version', '显示版本号', () => {
  displayVersion();
});

_commander.default.option('-V', '显示版本号', () => {
  displayVersion();
});

_commander.default.option('--no-cache', '对于引用到的文件，即使无改动也会再次编译');

_commander.default.option('-w, --watch', '监听文件改动');

_commander.default.option('-e, --env <env>', '配置环境', /^(production|development)$/i, 'development');

_commander.default.command('build').description('编译项目').action(projectPath => {
  if (!_util.default.isDir(_path.default.join(_util.default.currentDir, 'src'))) {
    _util.default.error('请先cd至项目根目录后再执行', '错误');
  } else if (!_util.default.isDir(_path.default.join(_util.default.currentDir, 'node_modules'))) {
    _util.default.error('请先执行npm install安装所需依赖', '错误');
  } else {
    console.warn('请使用--env代替-p --production配置, 支持production|development');

    if (!_commander.default.env) {
      _commander.default.env = 'development';
    }

    if (!_commander.default.cache || _commander.default.env === 'production') {
      _util.default.clearDist(() => {
        _compile.default.build(_commander.default);
      });
    } else {
      _compile.default.build(_commander.default);
    }
  }
});

_commander.default.command('new <projectName>').description('生成项目').action(name => {
  generateProject(name || 'wemixProject', _commander.default);
});

_commander.default.command('mkpage <pageName>').description('生成页面').action(name => {
  if (!_util.default.isDir(_path.default.join(_util.default.currentDir, 'src'))) {
    _util.default.error('请先cd至项目根目录后再执行', '错误');
  } else {
    generatePage(name, _commander.default);
  }
});

_commander.default.command('mkcomponent <componentName>').description('生成组件').action(name => {
  if (!_util.default.isDir(_path.default.join(_util.default.currentDir, 'src'))) {
    _util.default.error('请先cd至项目根目录后再执行', '错误');
  } else {
    generateComponent(name, _commander.default);
  }
});

_commander.default.command('mkbehavior <behaviorName>').description('生成Behavior').action(name => {
  if (!_util.default.isDir(_path.default.join(_util.default.currentDir, 'src'))) {
    _util.default.error('请先cd至项目根目录后再执行', '错误');
  } else {
    generateBehavior(name, _commander.default);
  }
});

_commander.default.parse(process.argv);